﻿using System;
using System.IO;
using System.Net;
using System.Text;
using System.Windows.Forms;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System.Collections.Generic;

namespace SpectacleTransformer {
    public partial class MainForm : Form {

        #region languages
        public static Language[] languages = new Language[] {
            new Language("Auto-detect", "auto"),
            new Language("Afrikaans", "af", "af"),
            new Language("Albanian", "sq", "sq"),
            new Language("Amharic", "am", "am"),
            new Language("Arabic", "ar", "ar"),
            new Language("Armenian", "hy", "hy"),
            new Language("Azerbaijani", "az", "az"),
            new Language("Basque", "eu", "eu"),
            new Language("Belarusian", "be", "be"),
            new Language("Bengali", "bn", "bn"),
            new Language("Bosnian", "bs", "bs"),
            new Language("Bulgarian", "bg", "bg"),
            new Language("Catalan", "ca", "ca"),
            new Language("Cebuano", "ceb", "ceb"),
            new Language("Chichewa", "ny", "ny"),
            new Language("Chinese", "zh", "zh"),
            new Language("Chinese (Traditional)", "zh_HANT", "zh_HANT"),
            new Language("Corsican", "co", "co"),
            new Language("Croatian", "hr", "hr"),
            new Language("Czech", "cs", "cs"),
            new Language("Danish", "da", "da"),
            new Language("Dutch", "nl", "nl"),
            new Language("English", "en", "en"),
            new Language("Esperanto", "eo", "eo"),
            new Language("Estonian", "et", "et"),
            new Language("Filipino", "tl", "tl"),
            new Language("Finnish", "fi", "fi"),
            new Language("French", "fr", "fr"),
            new Language("Frisian", "fy", "fy"),
            new Language("Galician", "gl", "gl"),
            new Language("Georgian", "ka", "ka"),
            new Language("German", "de", "de"),
            new Language("Greek", "el", "el"),
            new Language("Gujarati", "gu", "gu"),
            new Language("Haitian Creole", "ht", "ht"),
            new Language("Hausa", "ha", "ha"),
            new Language("Hawaiian", "haw", "haw"),
            new Language("Hebrew", "iw", "iw"),
            new Language("Hindi", "hi", "hi"),
            new Language("Hmong", "hmn", "hmn"),
            new Language("Hungarian", "hu", "hu"),
            new Language("Icelandic", "is", "is"),
            new Language("Igbo", "ig", "ig"),
            new Language("Indonesian", "id", "id"),
            new Language("Irish", "ga", "ga"),
            new Language("Italian", "it", "it"),
            new Language("Japanese", "ja", "ja"),
            new Language("Javanese", "jw", "jw"),
            new Language("Kannada", "kn", "kn"),
            new Language("Kazakh", "kk", "kk"),
            new Language("Khmer", "km", "km"),
            new Language("Kinyarwanda", "rw", "rw"),
            new Language("Korean", "ko", "ko"),
            new Language("Kurdish (Kurmanji)", "ku", "ku"),
            new Language("Kyrgyz", "ky", "ky"),
            new Language("Lao", "lo", "lo"),
            new Language("Latin", "la", "la"),
            new Language("Latvian", "lv", "lv"),
            new Language("Lithuanian", "lt", "lt"),
            new Language("Luxembourgish", "lb", "lb"),
            new Language("Macedonian", "mk", "mk"),
            new Language("Malagasy", "mg", "mg"),
            new Language("Malay", "ms", "ms"),
            new Language("Malayalam", "ml", "ml"),
            new Language("Maltese", "mt", "mt"),
            new Language("Maori", "mi", "mi"),
            new Language("Marathi", "mr", "mr"),
            new Language("Mongolian", "mn", "mn"),
            new Language("Myanmar (Burmese)", "my", "my"),
            new Language("Nepali", "ne", "ne"),
            new Language("Norwegian", "no", "no"),
            new Language("Odia (Oriya)", "or", "or"),
            new Language("Pashto", "ps", "ps"),
            new Language("Persian", "fa", "fa"),
            new Language("Polish", "pl", "pl"),
            new Language("Portuguese", "pt", "pt"),
            new Language("Punjabi", "pa", "pa"),
            new Language("Romanian", "ro", "ro"),
            new Language("Russian", "ru", "ru"),
            new Language("Samoan", "sm", "sm"),
            new Language("Scots Gaelic", "gd", "gd"),
            new Language("Serbian", "sr", "sr"),
            new Language("Sesotho", "st", "st"),
            new Language("Shona", "sn", "sn"),
            new Language("Sindhi", "sd", "sd"),
            new Language("Sinhala", "si", "si"),
            new Language("Slovak", "sk", "sk"),
            new Language("Slovenian", "sl", "sl"),
            new Language("Somali", "so", "so"),
            new Language("Spanish", "es", "es"),
            new Language("Sundanese", "su", "su"),
            new Language("Swahili", "sw", "sw"),
            new Language("Swedish", "sv", "sv"),
            new Language("Tajik", "tg", "tg"),
            new Language("Tamil", "ta", "ta"),
            new Language("Tatar", "tt", "tt"),
            new Language("Telugu", "te", "te"),
            new Language("Thai", "th", "th"),
            new Language("Turkish", "tr", "tr"),
            new Language("Turkmen", "tk", "tk"),
            new Language("Ukrainian", "uk", "uk"),
            new Language("Urdu", "ur", "ur"),
            new Language("Uyghur", "ug", "ug"),
            new Language("Uzbek", "uz", "uz"),
            new Language("Vietnamese", "vi", "vi"),
            new Language("Welsh", "cy", "cy"),
            new Language("Xhosa", "xh", "xh"),
            new Language("Yiddish", "yi", "yi"),
            new Language("Yoruba", "yo", "yo"),
            new Language("Zulu", "zu", "zu")
        };
        #endregion

        /// <summary>
        /// Whether or not the specified instance URL is valid.
        /// </summary>
        private bool validInstance = false;

        public MainForm() {
            InitializeComponent();
        }

        [STAThread]
        static void Main() {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            Application.Run(new MainForm());
        }

        private bool validateUri(string uriString) {
            try {
                Uri uri = new Uri(uriString, UriKind.Absolute);
                return uri.Scheme == Uri.UriSchemeHttp || uri.Scheme == Uri.UriSchemeHttps;
            } catch (UriFormatException) {
                return false;
            }
        }

        private void updateButtonStates() {
            translateButton.Enabled = sourceLanguagePicker.SelectedItem != null
                && destinationLanguagePicker.SelectedItem != null
                && validInstance
                && sourceTextBox.Text.Length > 0;
        }

        private void MainForm_Load(object sender, EventArgs e) {
            instanceUrlPicker.BeginUpdate();
            foreach (string item in Properties.Settings.Default.KnownInstances) {
                instanceUrlPicker.Items.Add(item);
            }
            if (instanceUrlPicker.Items.Contains(Properties.Settings.Default.LastSelectedInstance)) {
                instanceUrlPicker.SelectedItem = Properties.Settings.Default.LastSelectedInstance;
                validInstance = validateUri(instanceUrlPicker.SelectedItem.ToString());
                updateButtonStates();
            }
            instanceUrlPicker.EndUpdate();

            sourceLanguagePicker.BeginUpdate();
            destinationLanguagePicker.BeginUpdate();
            sourceLanguagePicker.Items.Clear();
            destinationLanguagePicker.Items.Clear();
            foreach (Language language in languages) {
                if (language.SourceName != null) sourceLanguagePicker.Items.Add(language);
                if (language.TargetName != null) destinationLanguagePicker.Items.Add(language);
            }
            sourceLanguagePicker.EndUpdate();
            destinationLanguagePicker.EndUpdate();
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e) {
            Properties.Settings.Default.Save();
        }

        private void instanceUrlPicker_TextChanged(object sender, EventArgs e) {
            validInstance = validateUri(instanceUrlPicker.Text);
            if (validInstance && instanceUrlPicker.SelectedItem != null)
                Properties.Settings.Default.LastSelectedInstance = instanceUrlPicker.SelectedItem.ToString();
            updateButtonStates();
        }

        private void sourceLanguagePicker_TextChanged(object sender, EventArgs e) {
            updateButtonStates();
        }

        private void destinationLanguagePicker_TextChanged(object sender, EventArgs e) {
            updateButtonStates();
        }

        private void sourceTextBox_TextChanged(object sender, EventArgs e) {
            updateButtonStates();
        }

        private void clearButton_Click(object sender, EventArgs e) {
            sourceTextBox.Clear();
            destinationTextBox.Clear();
        }

        private void translateButton_Click(object sender, EventArgs e) {
            Enabled = false;
            Uri hostUri = new Uri(instanceUrlPicker.Text);
            HttpWebResponse response = null;
            try {
                HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(new Uri(
                    hostUri,
                    String.Format(
                        "api/v1/{0}/{1}/{2}",
                        Uri.EscapeUriString(((Language)sourceLanguagePicker.SelectedItem).SourceName),
                        Uri.EscapeUriString(((Language)destinationLanguagePicker.SelectedItem).TargetName),
                        Uri.EscapeUriString(sourceTextBox.Text)
                    )
                ));
                request.UserAgent = String.Format("{0}/{1}", Application.ProductName, Application.ProductVersion);

                response = (HttpWebResponse)request.GetResponse();

                string responseBody;
                using (Stream stream = response.GetResponseStream()) {
                    StreamReader reader = new StreamReader(stream, Encoding.UTF8);
                    responseBody = reader.ReadToEnd();
                }

                if (response.StatusCode == HttpStatusCode.OK || response.StatusCode == HttpStatusCode.Created) {
                    string translation = null;
                    try {
                        JsonConvert.DeserializeObject<Dictionary<string, string>>(responseBody).TryGetValue("translation", out translation);
                    } catch (Exception) {}

                    if (String.IsNullOrEmpty(translation)) {
                        MessageBox.Show(
                            "The API request was performed successfully, but the server did not return a valid translation payload. Response payload:\r\n" + responseBody,
                            "Unexpected API response",
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Error
                        );
                        return;
                    }

                    // Save an unknown host when the request was completed
                    string uriString = hostUri.ToString();
                    if (!instanceUrlPicker.Items.Contains(uriString)) {
                        instanceUrlPicker.Items.Add(uriString);
                        instanceUrlPicker.SelectedItem = uriString;
                        Properties.Settings.Default.KnownInstances.Add(uriString);
                    }

                    destinationTextBox.Text = translation;
                } else {
                    string errorMessage;
                    try {
                        if (!JsonConvert.DeserializeObject<Dictionary<string, string>>(responseBody).TryGetValue("errorMsg", out errorMessage)) {
                            errorMessage = responseBody;
                        }
                    } catch (Exception) {
                        errorMessage = responseBody;
                    }

                    MessageBox.Show(
                        String.Format(
                            "An error occured while translating (HTTP {0} {1}):\r\n{2}",
                            response.StatusCode,
                            response.StatusDescription,
                            errorMessage
                        ),
                        "API error",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error
                    );
                }
            } catch (WebException ex) {
                MessageBox.Show(
                    "An error occured while running the API request:\r\n" + ex.ToString(),
                    "API request error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error
                );
            } finally {
                if (response != null) response.Close();
                Enabled = true;
            }
        }
    }
}
